;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; test boot loader for x86
   
;;; constants
boot_base       equ 0x07C0      ;; the segment base:offset pair for the
boot_offset     equ 0x0000      ;; boot code entrypoint

stack_segment   equ 0x9000        
stack_top       equ 0xFFFC

        
VBIOS           equ 0x10        ;; BIOS interrupt vector for video services
block_write     equ 0x09        ;; VBIOS routine - write a fixed number of times to the screen
ttype           equ 0x0E        ;; VBIOS routine - print character, teletype mode

;;; character constants
NULL            equ 0x00        ;; end of string marker
CR              equ 0x0D        ;; carriage return
LF              equ 0x0A        ;; line feed 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; macros
;
%define zero(x) xor x, x

%macro write 1
        mov si, %1
        call printstr
%endmacro
        
        
[bits 16]
[org boot_offset]
[section text]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; entry - the entrypoint to the code. Make a short jump past the BPB.
entry:
        jmp short redirect
        nop

redirect:       
         jmp boot_base:start

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; start
;;; This is the real begining of the code. The first order of
;;; business is clearing the interrupts, then setting the
;;; segment registers and the stack pointer.  

start:
        mov ax, stack_segment
        cli                     ; clear ints out of an abundance of caution
        mov ss, ax              ; set to match the code segment
        mov sp, stack_top       ; put the stack pointer to the top of SS
        sti                     ; reset ints so BIOS calls can be used
        mov ax, cs
        mov ds, ax              ; set DS == CS

;;;  hex printing demo - prints from FF down to zero
        ;; just print a prologue to show it is running
        write start_string
        
        ;;  init the loop and the string pointers
        mov cx, 0xff
        mov ax, hex_string
        mov di, ax
        mov si, ax
        
print_countdown:
        mov bl, cl
        call byte2hex
        write hex_string
        write comma
        
        loop print_countdown

        zero(bl)
        call byte2hex
        write hex_string
        
        ;; epilogue to show it didn't hang
        write end_string
        
halted:
        hlt
        jmp short halted
   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Auxilliary functions      

;;; printstr - prints the string point to by SI

printstr:
        push ax
        mov ah, ttype        ; set function to 'teletype mode'
  .print_char:   
        lodsb               ; update byte to print
        cmp al, NULL        ; test that it isn't NULL
        jz short .endstr
        int  VBIOS          ; put character in AL at next cursor position
        jmp short .print_char
  .endstr:
        pop ax
        ret


byte2hex:
        push cx
        push di
        
        mov cx, 2
        mov al, bl
        mov ah, bl
        and al, 0x0f            ; isolate low nibble
        shr ah, 4               ; isolate high nibble into low nibble
  .nibble_loop:
        cmp ah, 9
        jg .alphanum
	add ah, 0x30
	jmp short .store_char

  .alphanum:
        add ah, (0x41 - 0xA) ; NASM will compute imm. value
  
  .store_char:
        mov [di], ah
        mov ah, al
        inc di
        loop .nibble_loop        

        pop di
        pop cx
        ret
        
        
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; [section data]
        start_string db "Counting: ", CR, LF, NULL
        hex_string db 0, 0,  NULL
        comma db ", ", NULL
        end_string db CR, LF, "done.", CR, LF, NULL
        
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; pad out to 510, and then add the last two bytes needed for a boot disk
space   times ((0x0200 - 2) - ($ - $$)) db 0
bootsig   dw 0xAA55
